/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <system_settings.h>
#include <efl_extension.h>
#include <dlog.h>
#include <service_app.h>
#include <sync_adapter.h>
#include <stdlib.h>
#include <message_port.h>
#include "main.h"


int service_port_id = -1;

/**
 * @brief Callback function for Sync Adapter's start sync request.
 * @remarks This function matches the sync_adapter_start_sync_cb() type signature
 *          defined in the EFL API.
 *
 * @param account An account handle on which sync operation was requested
 *                or @c NULL in the case of accountless sync operation
 * @param sync_job_name A string representing a sync job which has been operated
 *                      or @c NULL in the case of data change sync operation (not used here)
 * @param sync_capability A string representing a sync job which has been operated
 *                        because of data change or @c NULL in the case of on demand
 *                        and periodic sync operation (not used here)
 * @param sync_job_user_data User data which contains additional information
 *                           related registered sync job (not used here)
 */
static bool on_start_cb(account_h account, const char *sync_job_name, const char *sync_capability,
                 bundle *sync_job_user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "*************************** on_start_cb ***************************");
    int val = -1;
    int ret = bundle_get_byte(sync_job_user_data, "string", (void **) &val, NULL);
    if (ret != BUNDLE_ERROR_NONE || val == -1) {
        dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_get_str() failed : %s", get_error_message(ret));
        return true;
    }

    dlog_print(DLOG_DEBUG, LOG_TAG, "Received string: %d", val);
    char *msg_to_send = NULL;

    if (val == ON_DEMAND_JOB_DONE)
        msg_to_send = "Service >>> Sync on demand is done";
    else if (val == PERIODIC_JOB_DONE)
        msg_to_send = "Service >>> Periodic sync is done";
    else if (val == SYNC_JOB_NOW)
        msg_to_send = "Service >>> Calendar data are synced now";

    bool exists = false;
    ret = message_port_check_trusted_remote_port(SYNC_ADAPTER_APP_ID, REMOTE_PORT, &exists);
    if (ret == MESSAGE_PORT_ERROR_NONE && exists) {
        bundle *b = bundle_create();
        ret = get_last_result();
        if (ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_create() failed : %s", get_error_message(ret));
            return true;
        }

        ret = bundle_add_str(b, "message", msg_to_send);
        if (b == NULL || ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_add_str() failed : %s", get_error_message(ret));
            return true;
        }

        ret = message_port_send_trusted_message(SYNC_ADAPTER_APP_ID, REMOTE_PORT, b);
        if (ret != MESSAGE_PORT_ERROR_NONE)
            dlog_print(DLOG_DEBUG, LOG_TAG, "message_port_send_trusted_message() failed : %s",
                       get_error_message(ret));

        bundle_free(b);
    }

    return true;
}

/**
 * @brief Callback function for Sync Adapter's cancel sync request.
 * @remarks This function matches the sync_adapter_cancel_sync_cb() signature
 *          defined in the service_app.h header file.
 *
 * @param account An account handle on which sync operation was requested
 *                or @c NULL in the case of accountless sync operation
 * @param sync_job_name A string representing a sync job which has been operated
 *                      or @c NULL in the case of data change sync operation (not used here)
 * @param sync_capability A string representing a sync job which has been operated
 *                        because of data change or @c NULL in the case of on demand
 *                        and periodic sync operation (not used here)
 * @param sync_job_user_data User data which contains additional information
 *                           related registered sync job (not used here)
 */
static void on_cancel_cb(account_h account, const char *sync_job_name, const char *sync_capability,
                  bundle *sync_job_user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "*************************** on_cancel_cb **************************");
}

/**
 * @brief Called when a trusted message is received.
 * @remarks This function matches the message_port_trusted_message_cb() signature
 *          defined in the service_app.h header file.
 *
 * @param trusted_local_port_id The message port ID returned by message_port_register_trusted_local_port()
 * @param remote_app_id The ID of the remote application that sent this message (not used here)
 * @param remote_port The name of the remote message port (not used here)
 * @param trusted_remote_port If @c true the remote port is a trusted port,
 *                            otherwise if @c false it is not
 * @param message The message passed from the remote application
 * @param user_data The user data passed from the register function (not used here)
 */
static void on_message(int trusted_local_port_id, const char *remote_app_id, const char *remote_port,
                bool trusted_remote_port, bundle *message, void *user_data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG,
               "***************************  on_message  **************************");

    char *val = NULL;
    bundle_get_str(message, "message", &val);
    dlog_print(DLOG_DEBUG, LOG_TAG, "Received string: %s", val);

    if (!strcmp(val, "exit"))
        service_app_exit();
}

/*
 * @brief Called at the start of the application.
 * @remarks This function matches the service_app_create_cb() signature
 *          defined in the service_app.h header file.
 *
 * @param[in] data The data passed to the callback (not used here)
 *
 * @return Always @c true, so that the application launches even in case of errors
 *         and displays the corresponding error messages
 */
static bool service_app_create(void *data)
{
    int ret = sync_adapter_set_callbacks((sync_adapter_start_sync_cb) on_start_cb,
                                         (sync_adapter_cancel_sync_cb) on_cancel_cb);
    dlog_print(DLOG_DEBUG, LOG_TAG, "> > > callbacks set: %d", ret);
    service_port_id =
        message_port_register_trusted_local_port(SELF_PORT,
                                                 (message_port_trusted_message_cb) on_message,
                                                 NULL);

    bool exists = false;
    ret = message_port_check_trusted_remote_port(SYNC_ADAPTER_APP_ID, REMOTE_PORT, &exists);
    if (ret == MESSAGE_PORT_ERROR_NONE && exists) {
        bundle *b = bundle_create();
        ret = get_last_result();
        if (ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_create() failed : %s", get_error_message(ret));
            return true;
        }

        ret = bundle_add_str(b, "message", "Service >>> Service has been started");
        if (b == NULL || ret != BUNDLE_ERROR_NONE) {
            dlog_print(DLOG_DEBUG, LOG_TAG, "bundle_add_str() failed : %s", get_error_message(ret));
            return true;
        }

        ret = message_port_send_trusted_message(SYNC_ADAPTER_APP_ID, REMOTE_PORT, b);
        if (ret != MESSAGE_PORT_ERROR_NONE)
            dlog_print(DLOG_DEBUG, LOG_TAG, "message_port_send_trusted_message() failed : %s",
                       get_error_message(ret));

        bundle_free(b);
    }

    return true;
}

/*
 * @brief Called once after the main loop of the application exits.
 * @details In this function the deinitialization code should be invoked and all
 *          the resources used by the application should be released if needed.
 * @remarks This function matches the service_app_terminate_cb() signature
 *          defined in the service_app.h header file.
 *
 * @param[in] data The data passed to the callback (not used here)
 */
static void service_app_terminate(void *data)
{
    sync_adapter_unset_callbacks();
    dlog_print(DLOG_DEBUG, LOG_TAG, "< < < callbacks unset.");
    return;
}

/**
 * @brief Creates and posts an ongoing notification.
 * @details Called when the service application receives an app control request
 *        (also called upon application launch).
 * @remarks This function matches the service_app_control_cb() type signature
 *          defined in the service_app.h header file.
 *
 * @param app_control[in] The handle to the app_control
 * @param data[in] The user data passed from the callback registration function
 *             (not used here)
 */
static void service_app_control(app_control_h app_control, void *data)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "service_app_control called");

    char *app_id = NULL;
    app_control_get_app_id(app_control, &app_id);

    if (app_id != NULL) {
        dlog_print(DLOG_DEBUG, LOG_TAG, "caller %s", app_id);
        free(app_id);
    }

    return;
}

/**
 * @brief This function will be called when the language is changed.
 */
static void service_app_lang_changed(app_event_info_h event_info, void *user_data)
{
    /* APP_EVENT_LANGUAGE_CHANGED */
    char *locale = NULL;

    system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

    if (locale != NULL) {
        elm_language_set(locale);
        free(locale);
    }
    return;
}


/**
 * @brief Main function of the application.
 */
int main(int argc, char *argv[])
{
    int ret;
    service_app_lifecycle_callback_s event_callback;
    app_event_handler_h handlers[5] = {NULL, };

    event_callback.create = service_app_create;
    event_callback.terminate = service_app_terminate;
    event_callback.app_control = service_app_control;

    /*
     * If you want to handle more events,
     * please check the application life cycle guide.
     */
    service_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, service_app_lang_changed, NULL);

    ret = service_app_main(argc, argv, &event_callback, NULL);
    if (ret != APP_ERROR_NONE)
        dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() failed. err = %d", ret);

    return ret;
}
